"use strict";
/*!
 * Copyright (c) 2017-present Cliqz GmbH. All rights reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
exports.__esModule = true;
var tslib_1 = require("tslib");
var tldts_experimental_1 = require("tldts-experimental");
var tokens_buffer_1 = tslib_1.__importDefault(require("./tokens-buffer"));
var utils_1 = require("./utils");
var TLDTS_OPTIONS = {
    extractHostname: true,
    mixedInputs: false,
    validateHostname: false
};
var TOKENS_BUFFER = new tokens_buffer_1["default"](300);
var Request = /** @class */ (function () {
    function Request(_a) {
        var requestId = _a.requestId, tabId = _a.tabId, type = _a.type, domain = _a.domain, hostname = _a.hostname, url = _a.url, sourceDomain = _a.sourceDomain, sourceHostname = _a.sourceHostname;
        this.id = requestId;
        this.tabId = tabId;
        this.type = type;
        this.url = url;
        this.hostname = hostname;
        this.domain = domain;
        this.sourceHostname = sourceHostname;
        this.sourceDomain = sourceDomain;
        this.sourceHostnameHash = utils_1.fastHash(this.sourceHostname);
        this.sourceDomainHash = utils_1.fastHash(this.sourceDomain);
        // Decide on party
        this.isThirdParty = this.sourceDomain.length === 0 ? false : this.sourceDomain !== this.domain;
        this.isFirstParty = !this.isThirdParty;
        // Check protocol
        this.isSupported = true;
        if (this.type === 'websocket' ||
            this.url.startsWith('ws:') ||
            this.url.startsWith('wss:')) {
            this.isHttp = false;
            this.isHttps = false;
            this.type = 'websocket';
            this.isSupported = true;
        }
        else if (this.url.startsWith('http:')) {
            this.isHttp = true;
            this.isHttps = false;
        }
        else if (this.url.startsWith('https:')) {
            this.isHttps = true;
            this.isHttp = false;
        }
        else {
            this.isHttp = false;
            this.isHttps = false;
            this.isSupported = false;
        }
        // Lazy attributes
        this.tokens = undefined;
        this.fuzzySignature = undefined;
    }
    /**
     * Create an instance of `Request` from raw request details.
     */
    Request.fromRawDetails = function (_a) {
        var _b = _a.requestId, requestId = _b === void 0 ? '0' : _b, _c = _a.tabId, tabId = _c === void 0 ? 0 : _c, _d = _a.url, url = _d === void 0 ? '' : _d, hostname = _a.hostname, domain = _a.domain, _e = _a.sourceUrl, sourceUrl = _e === void 0 ? '' : _e, sourceHostname = _a.sourceHostname, sourceDomain = _a.sourceDomain, _f = _a.type, type = _f === void 0 ? 'main_frame' : _f;
        url = url.toLowerCase();
        if (hostname === undefined || domain === undefined) {
            var parsed = tldts_experimental_1.parse(url, TLDTS_OPTIONS);
            hostname = hostname || parsed.hostname || '';
            domain = domain || parsed.domain || '';
        }
        // Initialize source URL
        if (sourceHostname === undefined || sourceDomain === undefined) {
            var parsed = tldts_experimental_1.parse(sourceUrl, TLDTS_OPTIONS);
            sourceHostname = sourceHostname || parsed.hostname || '';
            sourceDomain = sourceDomain || parsed.domain || '';
        }
        // source URL
        return new Request({
            requestId: requestId,
            tabId: tabId,
            domain: domain,
            hostname: hostname,
            url: url,
            sourceDomain: sourceDomain,
            sourceHostname: sourceHostname,
            sourceUrl: sourceUrl,
            type: type
        });
    };
    Request.prototype.getTokens = function () {
        if (this.tokens === undefined) {
            TOKENS_BUFFER.seekZero();
            if (this.sourceDomain) {
                TOKENS_BUFFER.push(utils_1.fastHash(this.sourceDomain));
            }
            if (this.sourceHostname) {
                TOKENS_BUFFER.push(utils_1.fastHash(this.sourceHostname));
            }
            utils_1.tokenizeInPlace(this.url, TOKENS_BUFFER);
            this.tokens = TOKENS_BUFFER.slice();
        }
        return this.tokens;
    };
    Request.prototype.getFuzzySignature = function () {
        if (this.fuzzySignature === undefined) {
            this.fuzzySignature = utils_1.createFuzzySignature(this.url);
        }
        return this.fuzzySignature;
    };
    Request.prototype.isMainFrame = function () {
        return this.type === 'main_frame' || this.type === 'mainFrame';
    };
    Request.prototype.isSubFrame = function () {
        return this.type === 'sub_frame' || this.type === 'subFrame';
    };
    return Request;
}());
exports["default"] = Request;
/**
 * Kept for backward compatibility. The recommended way is to call
 * `Request.fromRawDetails` directly.
 */
function makeRequest(details) {
    return Request.fromRawDetails(details);
}
exports.makeRequest = makeRequest;
//# sourceMappingURL=request.js.map