/*!
 * Copyright (c) 2017-present Cliqz GmbH. All rights reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
/// <reference types="chrome" />
export declare type ElectronRequestType = 'image' | 'mainFrame' | 'object' | 'other' | 'script' | 'stylesheet' | 'subFrame' | 'xhr';
export declare type PuppeteerRequestType = 'document' | 'stylesheet' | 'image' | 'media' | 'font' | 'script' | 'texttrack' | 'xhr' | 'fetch' | 'eventsource' | 'websocket' | 'manifest' | 'other';
export declare type WebRequestTypeChrome = chrome.webRequest.ResourceType;
export declare type WebRequestTypeFirefox = 'beacon' | 'csp_report' | 'font' | 'image' | 'imageset' | 'main_frame' | 'media' | 'object' | 'object_subrequest' | 'other' | 'ping' | 'script' | 'speculative' | 'stylesheet' | 'sub_frame' | 'web_manifest' | 'websocket' | 'xbl' | 'xml_dtd' | 'xmlhttprequest' | 'xslt';
export declare type WebRequestType = WebRequestTypeChrome | WebRequestTypeFirefox;
export declare type RequestType = WebRequestType | ElectronRequestType | PuppeteerRequestType;
export interface IRequestInitialization {
    requestId: string;
    tabId: number;
    url: string;
    hostname: string;
    domain: string;
    sourceUrl: string;
    sourceHostname: string;
    sourceDomain: string;
    type: RequestType;
}
export default class Request {
    /**
     * Create an instance of `Request` from raw request details.
     */
    static fromRawDetails({ requestId, tabId, url, hostname, domain, sourceUrl, sourceHostname, sourceDomain, type, }: Partial<IRequestInitialization>): Request;
    readonly type: RequestType;
    readonly isHttp: boolean;
    readonly isHttps: boolean;
    readonly isSupported: boolean;
    readonly isFirstParty: boolean;
    readonly isThirdParty: boolean;
    readonly id: string;
    readonly tabId: number;
    readonly url: string;
    readonly hostname: string;
    readonly domain: string;
    readonly sourceHostname: string;
    readonly sourceHostnameHash: number;
    readonly sourceDomain: string;
    readonly sourceDomainHash: number;
    private tokens;
    private fuzzySignature;
    constructor({ requestId, tabId, type, domain, hostname, url, sourceDomain, sourceHostname, }: IRequestInitialization);
    getTokens(): Uint32Array;
    getFuzzySignature(): Uint32Array;
    isMainFrame(): boolean;
    isSubFrame(): boolean;
}
/**
 * Kept for backward compatibility. The recommended way is to call
 * `Request.fromRawDetails` directly.
 */
export declare function makeRequest(details: Partial<IRequestInitialization>): Request;
